# Copyright (C) 2023-2024 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2024, Advanced Micro Devices, Inc."

import logging
from periphery import I2C

PM_LOWLEVEL_DEBUG = False

def pm_print(printstr):
    if(PM_LOWLEVEL_DEBUG):
        print(printstr)

logging.basicConfig(level=logging.ERROR)
class INA226(object):
    CONFIGURATION = 0x00
    SHUNT_VOLTAGE = 0x01
    BUS_VOLTAGE =   0x02
    POWER =         0x03
    CURRENT =       0x04
    CALIBRATION =   0x05
    MASK_ENABLE =   0x06
    ALERT_LIMIT =   0x07
    MANUFACTURER =  0XFE
    DIE_ID =        0XFF
    MANUFACTURER_ID_VAL = 0x5449
    DIE_ID_VAL =    0x2260

    die_id = 0
    manu_id = 0

    def __init__(self, address, devicepath):
        logging.info("Inside INA226 Constructor")
        self.addr = int(address, 0)
        self.i2c = I2C(devicepath)
        self._currentLSB = 0
        self._maxCurrent = 0
        self._shunt = 0
        self._phase_multiplier = 0
    
    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k in ['addr', 'i2c'] and v
        }
        return str(str_info)

    def initSensor(self, mc, sr, pm):
        self.die_id = self.getDieID()
        self.manu_id = self.getManufacturerID()
        if self.die_id == INA226.DIE_ID_VAL:
            if self.SetCalibration(mc, sr, pm):
                return True
            else:
                logging.error("INA226 Calibration Failed.")
                return False
        else:
            logging.error("INA226 init failed.")
            return False

    def getShuntVoltage(self):
        voltage = self._readRegister(INA226.SHUNT_VOLTAGE)
        voltage *= 2.5e-6
        return voltage

    def getBusVoltage(self):
        voltage = self._readRegister(INA226.BUS_VOLTAGE)
        voltage *= 1.25e-3
        return voltage

    def getPower(self):
        power = self._readRegister(INA226.POWER)
        power *= (self._currentLSB * 25)
        power *= self._phase_multiplier
        return power

    def getCurrent(self):
        value = self._readRegister(INA226.CURRENT)
        if value > 0x7fff:
            value -= 0x10000
            value = abs(value)

        current = self._currentLSB * value
        current *= self._phase_multiplier
        return current


    def SetCalibration(self, max_current, shunt_resistor, phase_multiplier):
        current_LSB = max_current / (32678 * 1000)
        calibration = (0.00512 * 1000000) / (current_LSB * shunt_resistor)

        if calibration > 32767:
            calibration = 32767

        self._writeRegister(INA226.CALIBRATION, int(calibration))
        calibration = self._readRegister(INA226.CALIBRATION)

        if calibration != 0:
            current_LSB = (0.00512 * 1000000) / (calibration * shunt_resistor)
        else:
            return False

        self._currentLSB = current_LSB
        self._maxCurrent = current_LSB * 32768
        self._shunt = shunt_resistor
        self._phase_multiplier = phase_multiplier
        return True

    def getManufacturerID(self):
        return self._readRegister(INA226.MANUFACTURER)

    def getDieID(self):
        return self._readRegister(int(INA226.DIE_ID))

    def readRegisterValues(self):
        registers = []
        for reg_adr in range(8):
            registers.append(self._readRegister(reg_adr))
        registers.append(self.manu_id)
        registers.append(self.die_id)
        for index, reg_val in enumerate(registers):
            pm_print("register[0x{0:02x}] = 0x{1:04x}".format(index, reg_val))
        return registers

    def writeRegisterValues(self, list_registers):
        for reg in list_registers:
            for k, v in reg.items():
                pm_print(f"{k}:{v}")
                #self._writeRegister(reg, v)
        pass

    def _readRegister(self, register):
        val = None
        try:
            msgs = [I2C.Message([register]), I2C.Message([0x00, 0x00], read=True)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}: 0x{1:02x}{2:02x}".format(self.addr, msgs[1].data[0], msgs[1].data[1]))
            val = (msgs[1].data[0] << 8) + msgs[1].data[1]
        except IOError:
                logging.error("INA226 _readRegister failed.")
        return val

    def _writeRegister(self, register, value):
        ret = False
        data = None
        try:
            data = bytearray(value.to_bytes(2, "big"))
            data.insert(0, register)
            msgs = [I2C.Message(data)]
            self.i2c.transfer(self.addr, msgs)
            ret = True
        except IOError:
            logging.error("INA226 _writeRegister failed.")
        return ret

    def __del__(self):
        logging.info("Inside INA226 Destructor")

class INA7XX(object):
    def __init__(self, address, devicepath):
        logging.info("Inside INA7XX Constructor")
        self.addr = int(address, 0)
        self.i2c = I2C(devicepath)
        pass

    def initSensor(self, mc, sr, pm):
        pass

    def getAddress(self):
        return self.addr

    def getShuntVoltage(self):
        pass
    def getBusVoltage(self):
        pass

    def getPower(self):
        pass

    def getCurrent(self):
        pass


    def SetCalibration(self, max_current, shunt_resistor, phase_multiplier):
        pass

    def getManufacturerID(self):
        pass

    def getDieID(self):
        pass

    def _readRegister(self, reg):
        val = None
        try:
            msgs = [I2C.Message([reg]), I2C.Message([0x00, 0x00], read=True)]
            self.i2c.transfer(self.addr, msgs)
            logging.debug("{0}: 0x{1:02x}{2:02x}".format(self.addr, msgs[1].data[0], msgs[1].data[1]))
            val = (msgs[1].data[0] << 8) + msgs[1].data[1]
        except IOError:
            logging.error("INA7XX _readRegister failed.")
        return val

    def _writeRegister(self, reg, val):
        ret = False
        data = None
        try:
            data = bytearray(val.to_bytes(2, "big"))
            data.insert(0, reg)
            msgs = [I2C.Message(data)]
            self.i2c.transfer(self.addr, msgs)
            ret = True
        except IOError:
            logging.error("INA7XX _writeRegister failed.")
        return ret

    def __del__(self):
        logging.info("Inside INA7XX Destructor")





# Copyright (C) 2023-2024 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2024, Advanced Micro Devices, Inc."

import os
import sys
import json
import logging
sys.path.append('../../utils')
sys.path.append('../../raft_services')
sys.path.append('../../raft_services/power_management')
import Pyro4
from periphery import I2C
from pm import PM
from utils import get_ip_and_port

RAFT_DIR = '/usr/share/raft/'

logging.basicConfig(level=logging.ERROR)

IPADDR, PORT = get_ip_and_port()
if len(IPADDR) == 0:
    print("CRITITCAL ERROR: Unable to Run Pyro Server.\n"
          "No eth0 network interface present.\n"
          "Please pass the ipaddress to __init__.py and retry.\n"
          "Usage:python3 __init__.py <ipaddress> <port>\n")
    sys.exit(1)
else:
    print("RAFT Pyro Server run successfully\n")

def exit_program():
    logging.error("CRITICAL ERROR: Unable to Run Raft-PM Server.")
    sys.exit(1)

def is_valid_json_file(file_path):
    if not os.path.isfile(file_path):
        logging.error("ERROR:root:Board Identification Json is not in Path : {file_path}")
        return False
    try:
        with open(file_path, 'r') as file:
            data = json.load(file)
            return True
    except json.JSONDecodeError:
        logging.error("ERROR:root:{file} Board Identification is not a JSON")
        return False

class board_eeprom:
    Name = ""
    I2C_Bus = ""
    I2C_Addr = ""

onboard = board_eeprom()

def parse_json_file(json_file):
    with open(json_file, 'r') as f:
        json_data = json.load(f)
        #print(json.dumps(json_data, indent=2))
    return json_data

def get_eeprom_data():
    result = False
    i2c = None
    onboard.Name = "Common"
    onboard.I2C_Bus = "/dev/i2c-1"
    onboard.I2C_Addr = 0x54
    i2c = I2C(onboard.I2C_Bus)
    if i2c is not None:
        data  = bytearray(256)
        try:
            msgs = [I2C.Message([0x0, 0x0]), I2C.Message(data, read=True)]
            i2c.transfer(onboard.I2C_Addr, msgs)
            eeprom_data = msgs[1].data
            result = True
        except IOError:
            logging.info(f"Onboard {onboard.Name} Eeprom read failed!")
        i2c.close()
    if result is False:
        onboard.Name = "Legacy"
        onboard.I2C_Bus = "/dev/i2c-11"
        onboard.I2C_Addr = 0x54
        i2c = I2C(onboard.I2C_Bus)
        if i2c is not None:
            data  = bytearray(256)
            try:
                msgs = [I2C.Message([0x0, 0x0]), I2C.Message(data, read=True)]
                i2c.transfer(onboard.I2C_Addr, msgs)
                eeprom_data = msgs[1].data
                result = True
            except IOError:
                logging.info(f"Onboard {onboard.Name} Eeprom read failed!")
            i2c.close()
        if result is False:
            logging.error("Board Eeprom Identification Failed.")
            exit_program()
    return eeprom_data

def get_product_name():
    eeprom_data = get_eeprom_data()
    offset = 0x15
    length = int.from_bytes(eeprom_data[offset:offset+1], "big") & 0x3f
    name = eeprom_data[offset+1:(offset+1 + length)].decode("utf-8").strip('\x00')
    return name


product_name = get_product_name()
json_file = os.path.join(RAFT_DIR, 'xserver/raft_services/power_management/board', '.'.join((product_name, 'json')))

if is_valid_json_file(json_file):
    json_data = parse_json_file(json_file)
    board_name = os.path.splitext(os.path.basename(json_file))
else:
    exit_program()

PM = Pyro4.expose(PM)

Pyro4.Daemon.serveSimple(
    {
        PM(json_data, board_name[0], onboard): "PM",
    },
    host=IPADDR,
    port=PORT,
    ns=False,
    verbose=True,
)



# Copyright (C) 2023-2024 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim, Sree Parvathi Anish"
__copyright__ = "Copyright 2023-2024, Advanced Micro Devices, Inc."

import os
import sys
import json
import argparse
import csv
import time
import datetime
sys.path.append('/usr/share/raft/xclient/raft_services')
from pm_client import PM_Client

def update_csv_data(csv_data, data, sample_rate, duration):
    # Convert the dictionary to a list
    columns_order = []
    row_item = {}
    write_timestamp = 1
    for k, domain_list in data.items():
        for index, domain in enumerate(domain_list):
            for key, value in domain.items():
                for rails_item in value["Rails"]:
                    for rails_key, rails_value in rails_item.items():
                        if write_timestamp == 1:
                            columns_order.append("Timestamp")
                            start_time = datetime.datetime.now()
                            row_item.update({"Timestamp" : start_time})
                            write_timestamp = 0
                        column_name = f"{key}-{rails_key}-{list(rails_value.keys())[0]}"
                        columns_order.append(column_name)
                        row_item.update({f"{column_name}": rails_value['Voltage']})
                        column_name = f"{key}-{rails_key}-{list(rails_value.keys())[2]}"
                        columns_order.append(column_name)
                        row_item.update({f"{column_name}": rails_value['Current']})
                        column_name = f"{key}-{rails_key}-{list(rails_value.keys())[3]}"
                        columns_order.append(column_name)
                        row_item.update({f"{column_name}": rails_value['Power']})
        column_name = f"{key}-TotalPower"
        columns_order.append(column_name)
        row_item.update({f"{column_name}": value["Total Power"]})
    csv_data.append(row_item)
    return csv_data, columns_order

def pm_csv_dump(csv_data, columns_order, filepath, filename):
    csv_file = f"{filepath}/{filename}"
    try:
        with open(csv_file, 'w') as csvfile:
            csv_writer = csv.DictWriter(csvfile, fieldnames=columns_order, lineterminator='\n')
            csv_writer.writeheader()
            for row in csv_data:
                    csv_writer.writerow(row)
            print(f"The requested data is written to file - {csv_file}")
    except IOError:
        print(f"I/O error while opening {csv_file} to write")

def main():
    client = PM_Client()

    parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
    parser.add_argument("-c", "--command", required=True, choices=["boardinfo", "domains", "powervalue", "rails", "raildetail", "railvalue", "loglevel", "pstemp", "sysmon", "stats",
                        "listpower", "getpower", "getcalpower", "getinaconf", "setinaconf",
                        "listvoltage", "enablevoltage", "disablevoltage", "getvoltage", "setvoltage", "setbootvoltage", "restorevoltage", "getregulator",
                        "output-csv"],
                        help="Specify the command (boardinfo, domains, rails or railvalue ...)")

    args, remaining_args = parser.parse_known_args()

    if args.command:
        if args.command == "rails":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetRailsOfDomain(target_args.target)))

        elif args.command == "railvalue":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetValueOfRail(target_args.target)))
        elif args.command == "raildetail":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetRailDetails(target_args.target)))

        elif args.command == "boardinfo":
            print(json.dumps(client.GetBoardInfo(), indent=2))

        elif args.command == "domains":
            print(json.dumps(client.GetPowerDomains(), indent=2))

        elif args.command == "powervalue":
            print(json.dumps(client.GetPowersAll(), indent=2))

        elif args.command == "loglevel":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=False, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target is None:
                print(json.dumps(client.logger.level))
            else:
                if target_args.target:
                    print(f"Target: {target_args.target}")

        elif args.command == "sysmon":
            print(json.dumps(client.GetSysmonTemperatures(), indent=2))

        elif args.command == "pstemp":
            print(json.dumps(client.GetPSTemperature(), indent=2))

        elif args.command == "stats":
            print(json.dumps(client.GetSystemStats(), indent=2))

        elif args.command == "listpower":
            print(json.dumps(client.ListPowerSensors(), indent=2))

        elif args.command == "getpower":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetPowerSensor(target_args.target), indent=2))

        elif args.command == "getcalpower":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetCalPowerSensor(target_args.target), indent=2))

        elif args.command == "getinaconf":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetPowerSensorConf(target_args.target), indent=2))

        elif args.command == "setinaconf":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                value_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
                value_parser.add_argument("-v", "--value", required=True, help="Specify the target")
                value_args, value_remining_args = value_parser.parse_known_args(target_remining_args)
                if value_args.value:
                    print(value_args.value)
                    conf_data = json.loads(value_args.value)
                    print(conf_data)
                    print(json.dumps(client.SetPowerSensorConf(target_args.target, conf_data), indent=2))

        elif args.command == "listvoltage":
            print(json.dumps(client.ListVoltages(), indent=2))

        elif args.command == "enablevoltage":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.EnableVoltage(target_args.target), indent=2))

        elif args.command == "disablevoltage":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.DisableVoltage(target_args.target), indent=2))

        elif args.command == "getvoltage":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetVoltage(target_args.target), indent=2))

        elif args.command == "setvoltage":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                value_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
                value_parser.add_argument("-v", "--value", required=True, help="Specify the target")
                value_args, value_remining_args = value_parser.parse_known_args(target_remining_args)
                if value_args.value:
                    print(json.dumps(client.SetVoltage(target_args.target, round(float(value_args.value), 3)), indent=2))

        elif args.command == "setbootvoltage":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                value_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
                value_parser.add_argument("-v", "--value", required=True, help="Specify the target")
                value_args, value_remining_args = value_parser.parse_known_args(target_remining_args)
                if value_args.value:
                    print(json.dumps(client.SetBootVoltage(target_args.target, round(float(value_args.value), 3)), indent=2))
        
        elif args.command == "restorevoltage":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                    print(json.dumps(client.RestoreVoltage(target_args.target), indent=2))
        
        elif args.command == "getregulator":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            if target_args.target:
                print(json.dumps(client.GetRegulatorAll(target_args.target), indent=2))

        elif args.command == "output-csv":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-d", "--duration", required=True, help="Specify the duration in seconds")
            target_parser.add_argument("-s", "--samplingrate", required=True, help="Specify the sampling rate")
            target_parser.add_argument("-o", "--outputfilepath", default='/home/petalinux', help="Specify the path to write the file")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)
            print(f"target_args.duration = {target_args.duration}, target_remining_args.samplingrate = {target_args.samplingrate}")
            sample_rate = int(target_args.samplingrate)
            filepath = target_args.outputfilepath
            duration = int(target_args.duration)
            # Check the input arguments
            if (sample_rate < 1 or sample_rate > 12):
                print(f"sample rate should be between 1 and 12")
                sys.exit(1)
            if (duration <= 0):
                print(f"duration should be a positive value")
                sys.exit(1)
            if not os.path.exists(filepath):
                print(f"The path {filepath} is valid.")
                sys.exit(1)

            # Find the sleep time from the sample rate
            sleeptime = 1/sample_rate

            csv_data = []
            # Mark the start time
            start_datetime = datetime.datetime.now()
            start_time = time.time()
            while True:
                operation_start_time = time.time()
                values = client.GetValuesAll()
                csv_data, columns_order = update_csv_data(csv_data, values, sample_rate, duration)
                operation_exec_time = time.time() - operation_start_time
                # If operation execution time is higher than sleep time, no need to sleep
                if (sleeptime - operation_exec_time) > 0:
                    time.sleep(sleeptime -  operation_exec_time)
                if time.time() - start_time >= duration:
                    break
            filename = start_datetime.strftime(f"pmtool_s-{sample_rate}_d-{duration}_%Y-%m-%d_%H-%M-%S.csv")
            pm_csv_dump(csv_data, columns_order, filepath, filename)

        else:
            args = parser.parse_args(remaining_args)

if __name__ == "__main__":
    # call main
    main()




# Copyright (C) 2023-2024 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2024, Advanced Micro Devices, Inc."

import os
import sys
import logging
import dbm.dumb


RAFT_DIR = '/usr/share/raft/'
sys.path.append(RAFT_DIR + 'xserver/utils')
sys.path.append(RAFT_DIR + 'xserver/raft_services/power_management/devices')

from pmic import PMIC
from devices.stats import Stats
from devices.sysmon import Sysmon
from utils import get_python_log_levels
from pm_types import *


class PM(object):
    logger = None
    domains = []
    sensors = []
    voltages = []
    pmic = None
    sysmons = []
    status = None
    pdi_file = ""

    def __init__(self, json_data, board_name, eeprom):
        self.logger = self.GetLogger()
        self.boardeeprom = eeprom
        self.board_name = board_name
        board_data = json_data[board_name]

        if 'FEATURE' in board_data:
            feature_list = board_data['FEATURE']['List']

        if 'powerdomain' in feature_list:
            if 'POWER DOMAIN' in board_data:
                for key, val in board_data['POWER DOMAIN'].items():
                    temp_d = Domain(**val)
                    for index, railname in enumerate(temp_d.railnames):
                        for k, v in board_data['POWER_SENSORS'].items():
                            if k == railname:
                                if temp_d.railnames[index] != v['Name']:
                                    temp_d.railnames[index] = v['Name']
                    self.domains.append(temp_d)

        if 'power' in feature_list:
            if 'POWER_SENSORS' in board_data:
                for k, v in board_data['POWER_SENSORS'].items():
                    temp_r = Rails(**v)
                    self.sensors.append(temp_r)

        if 'voltage' in feature_list:
            if 'VOLTAGE' in board_data:
                for k, v in board_data['VOLTAGE'].items():
                    tempVolt = Rails(**v)
                    self.voltages.append(tempVolt)

        if 'temp' in feature_list:
            if 'Temperature' in board_data:
                if "versal-isa-0000" in board_data['Temperature']['Sensor']:
                    try:
                        temp_s = Sysmon(None, None, Sysmon_Device_Type.SYSFS)
                        if temp_s is not None:
                            self.sysmons.append(temp_s)
                    except Exception as e:
                        print(e)
                elif "i2c" in board_data['Temperature']['Sensor']:
                    tokens = board_data['Temperature']['Sensor'].split("-")
                    address = tokens[3]
                    device_path = "/dev/i2c-" + tokens[2]
                    try:
                        temp_s = Sysmon(address, device_path, Sysmon_Device_Type.I2C)
                        if temp_s is not None:
                            self.sysmons.append(temp_s)
                    except Exception as e:
                        print(e)

        if 'Boot Config' in board_data:
            pdi_file = board_data['Boot Config']['PDI']

        try:
            self.status = Stats()
            if self.status is None:
                self.logger.error(f"PM: InitStats failed.")
        except Exception as e:
            self.logger.error(f"PM: InitStats failed.")
            self.exit_program()

        try:
            self.pmic = PMIC(self.boardeeprom, self.domains, self.sensors, self.voltages)
        except Exception as e:
            print(traceback.format_exc())
            self.exit_program()

        if self.pmic is None:
            self.logger.error(f"PM: InitPmic failed.")
        self.logger.info("Inside PM Constructor")

    @staticmethod
    def exit_program():
        logging.critical("CRITICAL ERROR: Unable to Run Pyro Server.\n")
        sys.exit(1)

    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger

        """
        log_level = logging.ERROR
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            logger.handler_set = True
            logger.disabled = False
        return logger

    # Log level
    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :param : None
        :return: Dictionary showing the log levels supported by logging library
        """
        return get_python_log_levels()

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"PythonLogLevel = {PythonLogLevel}")
        LogLevelsDict = get_python_log_levels()
        if PythonLogLevel == LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def GetBoardInfo(self):
        """
        Gets Board's Info

        :param : None
        :return: Board Info
        """
        return self.pmic.boardinfo

    def GetPSTemperature(self):
        """
        Gets SysCtl's Temperature Info

        :param : None
        :return: SysCtl's Temperature Info
        """
        return self.status.getTemperatures()

    def GetPowerDomains(self):
        """
        Gets list of Power Domains.

        :param : None
        :return: Domains
        """
        powerdomains = {}
        powerdomains['POWER DOMAINS'] = []
        for d in self.domains:
            powerdomains['POWER DOMAINS'].append({'name': d.name})
        return powerdomains

    def _find_domain(self, domainname):
        temp_d = None
        for d in self.domains:
            if d.name == domainname:
                temp_d = d
        return temp_d

    def GetRailsOfDomain(self, domainname):
        """
        Gets list of Rails given domain name.

        :param domainname: string of a "domainname"
        :return: Rails
        """
        data = {}
        domain = self._find_domain(domainname)
        if domain is None:
            data['Success'] = False
            data['Message'] = f'{domainname} does not exists'
        else:
            data[domainname] = []
            for name in domain.railnames:
                data[domainname].append({'name': name})
        return data

    def GetValueOfDomain(self, domainname):
        """
        Gets the domain's all rail sensor values given domain name.

        :param : string of a "domainname"
        :return: The domain's all rails sensor values of the Rail
        """
        total_power = 0.0
        data = {}
        domain = self._find_domain(domainname)
        if domain is None:
            data['Success'] = False
            data['Message'] = f'{domainname} does not exists'
        else:
            data[domainname] = {}
            data[domainname]['Rails'] = []
            for railname in domain.railnames:
                rail_value = self.GetPowerSensor(railname)
                data[domainname]['Rails'].append(rail_value)
                total_power += rail_value[railname]['Power']
            data[domainname]['Total Power'] = round(total_power, 4)
        return data

    def GetPowerValueOfDomain(self, domainname):
        """
        Gets the domain's power value given domain name.

        :param : string of a "domainname"
        :return: The domain's power value
        """
        domain_power = 0.0
        data = {}
        domain = self._find_domain(domainname)
        if domain is None:
            data['Success'] = False
            data['Message'] = f'{domainname} does not exists'
        else:
            data[domainname] = {}
            domain_power = 0.0
            for railname in domain.railnames:
                temp_ps = self.GetPowerSensor(railname)
                domain_power += temp_ps[railname]['Power']
                data[domainname]['Power'] = round(domain_power, 4)
        return data

    def GetPowersAll(self):
        """
        Gets the boards's all domain's and total power values

        :param : None
        :return: The boards's all domain's and total power values
        """
        data = {}
        data[self.board_name] = {}
        data[self.board_name]['Power Domains'] = []
        total_power = 0.0
        for domain in self.domains:
            temp_p = self.GetPowerValueOfDomain(domain.name)
            total_power += temp_p[domain.name]['Power']
            data[self.board_name]['Power Domains'].append(temp_p)
        data[self.board_name]['Total Power'] = round(total_power, 4)
        return data

    def GetValuesAll(self):
        """
        Gets the boards's all domain's rails sensor values

        :param : None
        :return: The board's all rails sensor values of the Rail
        """
        data = {}
        data[self.board_name] = []
        for domain in self.domains:
            data[self.board_name].append(self.GetValueOfDomain(domain.name))
        return data

    def GetSysmonTemperatures(self):
        data = {}
        if len(self.sysmons) == 0:
            data['Success'] = False
            data['Message'] = f"Sysmon is not available."
            return data
        else:
            list_s = []
            for s in self.sysmons:
                temperature, minimum, max_max, min_min = s.ReadSysmonTemperatures()
                data['TEMP'] = temperature
                data['MIN'] = minimum
                data['MAX_MAX'] = max_max
                data['MIN_MIN'] = min_min
                list_s.append(data)
            return list_s

    def GetSystemStats(self):
        data = {}
        data = self.status.getLastStatus()
        return data

    def ListPowerSensors(self):
        self.logger.info(f"ListPowerSensors()")
        data = {}
        list_ps = []
        for s in self.pmic.power_sensors:
            list_ps.append(s.name)
        data['power_sensors'] = list_ps
        return data

    def _find_power_sensor(self, sensor_name):
        temp_s = None
        for ps in self.pmic.power_sensors:
            if ps.name == sensor_name:
                temp_s = ps
        return temp_s
    
    def _error_message(message_str):
        data = {}
        data['Success'] = False
        data['Message'] = message_str

    def GetPowerSensor(self, sensor_name):
        self.logger.info(f"GetPowerSensor({sensor_name})")
        data = {}
        values = {}
        ps = self._find_power_sensor(sensor_name)
        if ps is None:
            data['Success'] = False
            data['Message'] = f'{sensor_name} does not exits'
            self.logger.error(f'GetPowerSensor({sensor_name}) does not exits')
        else:
            if ps._sensor is None:
                data['Success'] = False
                data['Message'] = f'{sensor_name} sensor is not defined'
                self.logger.error(f'GetPowerSensor({sensor_name}) sensor is not defined')
            else:
                vb, vs, i, p = self.pmic.GetSensorValues(ps._sensor)
                values['Voltage'] = round(vb, 4)
                values['Vshunt'] = round(vs, 4)
                values['Current'] = round(i, 4)
                values['Power'] = round(p, 4)
                data[sensor_name] = values
        return data

    def GetPowerCalSensor(self, sensor_name):
        data = {}
        values = {}
        ps = self._find_power_sensor(sensor_name)
        if ps is None:
            data['Success'] = False
            data['Message'] = f'{sensor_name} does not exits'
            self.logger.error(f'GetPowerCalSensor({sensor_name}) does not exits')
        else:
            if ps._sensor is None:
                data['Success'] = False
                data['Message'] = f'{sensor_name} sensor is not defined'
                self.logger.error(f'GetPowerCalSensor({sensor_name}) sensor is not defined')
            else:
                vb, vs, i, p = self.pmic.GetSensorValues(ps._sensor)
                values['voltage'] = round(vb, 4)
                values['vshunt'] = round(vs, 4)
                values['current'] = round(i, 4)
                values['power'] = round(p, 4)
                data[sensor_name] = values
        return data

    def GetPowerSensorConf(self, sensor_name):
        data = {}
        ps = self._find_power_sensor(sensor_name)
        if ps is None:
            data['Success'] = False
            data['Message'] = f'{sensor_name} does not exits'
            self.logger.error(f'GetPowerSensorConf({sensor_name}) does not exits')
        else:
            if ps._sensor is None:
                data['Success'] = False
                data['Message'] = f'{sensor_name} sensor is not defined'
                self.logger.error(f'GetPowerSensorConf({sensor_name}) sensor is not defined')
            else:
                data[sensor_name] = self.pmic.GetPowerSensorConf(ps._sensor)
        return data

    def SetPowerSensorConf(self, sensor_name, conf):
        data = {}
        ps = self._find_power_sensor(sensor_name)
        if ps is None:
            data['Success'] = False
            data['Message'] = f'{sensor_name} does not exits'
            self.logger.error(f'EnableVoltage({sensor_name}) does not exits')
        else:
            if ps._sensor is None:
                data['Success'] = False
                data['Message'] = f'{sensor_name} sensor is not defined'
                self.logger.error(f'SetPowerSensorConf({sensor_name}) sensor is not defined')
            else:
                self.pmic.SetPowerSensorConf(ps._sensor, conf)
                data['Success'] = True
        return data

    def ListVoltages(self):
        self.logger.info(f"ListVoltages")
        data = {}
        list_v = []
        for v in self.pmic.voltages:
            voltage = {}
            voltage[v.name] = {
                'typical_volt' : v.typical_volt
            }
            list_v.append(voltage)
        data['voltages'] = list_v
        return data

    def _find_voltage(self, voltage_name):
        temp_voltage = None
        for v in self.pmic.voltages:
            if v.name == voltage_name:
                temp_voltage = v
        return temp_voltage

    def EnableVoltage(self, voltage_name):
        self.logger.info(f"EnableVoltage({voltage_name})")
        data = {}
        v = self._find_voltage(voltage_name)
        if v is None:
            data['Success'] = False
            data['Message'] = f'{voltage_name} does not exits'
            self.logger.error(f'{voltage_name} does not exits')
        else:
            if v._output is None:
                data['Success'] = False
                data['Message'] = f'{voltage_name} regulator is not defined'
                self.logger.error(f'EnableVoltage({voltage_name}) regulator is not defined')
            else:
                self.pmic.EnableVoltage(v._output)
                data['Success'] = True
        return data

    def DisableVoltage(self, voltage_name):
        self.logger.info(f"DisableVoltage({voltage_name})")
        data = {}
        v = self._find_voltage(voltage_name)
        if v is None:
            data['Success'] = False
            data['Message'] = f'{voltage_name} does not exits'
            self.logger.error(f'{voltage_name} does not exits')
        else:
            if v._output is None:
                data['Success'] = False
                data['Message'] = f'{voltage_name} regulator is not defined'
                self.logger.error(f'DisableVoltage({voltage_name}) regulator is not defined')
            else:
                self.pmic.DisableVoltage(v._output)
                data['Success'] = True
        return data

    def GetRegulatorAll(self, voltage_name):
        self.logger.info(f"GetRegulatorAll({voltage_name})")
        data = {}
        v = self._find_voltage(voltage_name)
        if v is None:
            data['Success'] = False
            data['Message'] = f'{voltage_name} does not exits'
            self.logger.error(f'{voltage_name} does not exits')
        else:
            if v._output is None:
                data['Success'] = False
                data['Message'] = f'{voltage_name} regulator is not defined'
                self.logger.error(f'GetVoltage({voltage_name}) regulator is not defined')
            else:
                data[voltage_name] = self.pmic.GetRegulatorAll(v._output)
        return data

    def GetVoltage(self, voltage_name):
        self.logger.info(f"GetVoltage({voltage_name})")
        data = {}
        v = self._find_voltage(voltage_name)
        if v is None:
            data['Success'] = False
            data['Message'] = f'{voltage_name} does not exits'
            self.logger.error(f'{voltage_name} does not exits')
        else:
            if v._output is None:
                data['Success'] = False
                data['Message'] = f'{voltage_name} pmbus regulator is not defined'
                self.logger.error(f'GetVoltage({voltage_name}) pmbus regulator is not defined')
            else:
                data[voltage_name] = self.pmic.GetVoltage(v._output)
        return data

    def SetVoltage(self, voltage_name, new_value):
        self.logger.info(f"SetVoltage({voltage_name}, {new_value})")
        data = {}
        v = self._find_voltage(voltage_name)
        if v is None:
            data['Success'] = False
            data['Message'] = f'{voltage_name} does not exits'
            self.logger.error(f'SetVoltage({voltage_name}, {new_value}) {voltage_name} does not exits')
            return data
        else:
            if new_value < v.minimum_volt:
                data['Success'] = False
                data['Message'] = f'asked value({new_value}) below minimum({v.minimum_volt}) for {v.name}'
                self.logger.error(f'SetVoltage({voltage_name}, {new_value}) asked value below minimum({v.minimum_volt})')
                return data
            elif new_value > v.maximum_volt:
                data['Success'] = False
                data['Message'] = f'asked value({new_value}) above maximum({v.maximum_volt}) for {v.name}'
                self.logger.error(f'SetVoltage({voltage_name}, {new_value}) asked value above maximum({v.maximum_volt})')
                return data
            else:
                if v._output is None:
                    data['Success'] = False
                    data['Message'] = f'{voltage_name} pmbus regulator is not defined'
                    self.logger.error(f'GetVoltage({voltage_name}) pmbus regulator is not defined')
                    return data
                else:
                    self.pmic.SetVoltage(v._output, new_value)
                    data['Success'] = True
        return data

    def SetBootVoltage(self, voltage_name, boot_value):
        self.logger.info(f"SetBootVoltage({voltage_name}, {boot_value})")
        data = {}
        v = self._find_voltage(voltage_name)
        if v is None:
            data['Success'] = False
            data['Message'] = f'{voltage_name} does not exits'
            self.logger.error(f'SetBootVoltage({voltage_name}, {boot_value}) {voltage_name} does not exits')
            return data
        else:
            if boot_value < v.minimum_volt:
                data['Success'] = False
                data['Message'] = f'asked value({boot_value}) below minimum({v.minimum_volt}) for {v.name}'
                self.logger.error(f'SetBootVoltage({voltage_name}, {boot_value}) asked value({boot_value}) below minimum({v.minimum_volt})')
                return data
            elif boot_value > v.maximum_volt:
                data['Success'] = False
                data['Message'] = f'asked value({boot_value}) above maximum({v.maximum_volt}) for {v.name}'
                self.logger.error(f'SetBootVoltage({voltage_name}, {boot_value}) asked value({boot_value}) above maximum({v.maximum_volt})')
                return data
            else:
                if v._output is None:
                    data['Success'] = False
                    data['Message'] = f'{voltage_name} pmbus regulator is not defined'
                    self.logger.error(f'SetBootVoltage({voltage_name}, {boot_value}) pmbus regulator is not defined')
                    return data

        directory = os.path.dirname(RAFT_DIR + '.raft/')
        if directory and not os.path.exists(directory):
            os.makedirs(directory)
        bootvoltage_path = os.path.join(directory, 'bootvoltage')

        try:
            with dbm.dumb.open(bootvoltage_path, 'c') as db:
                db[voltage_name] = "{:.3f}".format(round(boot_value, 3))
                data['Success'] = True
        except Exception as err:
            self.logger.error(f'SetBootVoltage({voltage_name}, {boot_value}) unknown error as {err}')
            data['Success'] = False
            data['Message'] = f"Unknown error: {err}"
        finally:
            db.sync()
            db.close()
        return data

    def RestoreVoltage(self, voltage_name):
        self.logger.info(f"RestoreVoltage({voltage_name})")
        data = {}
        v = self._find_voltage(voltage_name)
        if v is None:
            data['Success'] = False
            data['Message'] = f'{voltage_name} does not exits'
            self.logger.error(f'RestoreVoltage({voltage_name}) {voltage_name} does not exits')
        else:
            self.SetVoltage(voltage_name, v.typical_volt)
            directory = os.path.dirname(RAFT_DIR + '.raft/')
            bootvoltage_path = os.path.join(directory, 'bootvoltage')
            try:
                with dbm.dumb.open(bootvoltage_path, 'w') as db:
                    del db[voltage_name]
                    data['Success'] = True
            except KeyError:
                self.logger.error(f"RestoreVoltage({voltage_name}) voltage is not in bootvoltage list")
                data['Success'] = False
                data['Message'] = f"{voltage_name} voltage is not in bootvoltage list"
            finally:
                db.sync()
                db.close()
        return data

    def __del__(self):
        self.logger.info("Inside PM Destructor")



# Copyright (C) 2023-2024 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023-2024, Advanced Micro Devices, Inc."

import os
import sys
import datetime
import logging
import dbm.dumb
from periphery import I2C
from utils import get_python_log_levels

RAFT_DIR = '/usr/share/raft/'
sys.path.append(RAFT_DIR + 'xserver/utils')
sys.path.append(RAFT_DIR + 'xserver/raft_services/power_management/devices')

from pm_types import *
from devices.sensors import *
from devices.regulators import *
class PMIC(object):
    logger = None
    domains = []
    voltages = []
    power_sensors = []
    boardinfo = {}

    def __init__(self, eeprom, pm_domains, pm_powersensor, pm_voltages):
        self.logger = self.GetLogger()
        self.domains = pm_domains

        if eeprom:
            i2c = I2C(eeprom.I2C_Bus)
            bytes = bytearray(256)
            try:
                msgs = [I2C.Message([0x0, 0x0]), I2C.Message(bytes, read=True)]
                i2c.transfer(eeprom.I2C_Addr, msgs)
                eeprom_data = msgs[1].data
                #print(''.join('{:02x} '.format(x) for x in eeprom_data))
                offset = 0xA
                self.boardinfo["Language"] = eeprom_data[offset]

                if True: #isProdBoard
                    self.boardinfo["Silicon Revision"] = "PROD"
                else:
                    self.boardinfo["Silicon Revision"] = ""

                build_date = datetime.datetime(1996, 1, 1)
                minutes  = (eeprom_data[0xd] << 16 | eeprom_data[0xc] << 8 | eeprom_data[0xb])
                time_delta = datetime.timedelta(minutes=minutes)
                build_date += time_delta
                time_string = build_date.strftime('%c')
                self.boardinfo['Manufacturing Date'] = time_string

                offset = 0xe
                length = int.from_bytes(eeprom_data[offset:offset+1], "big")
                length &= 0x3f
                self.boardinfo["Manufacturer"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')

                offset = offset + length + 1
                length = int.from_bytes(eeprom_data[offset:offset+1], "big")
                length &= 0x3f
                self.boardinfo["Product Name"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')

                offset = offset + length + 1
                length = int.from_bytes(eeprom_data[offset:offset+1], "big")
                length &= 0x3f
                self.boardinfo["Board Serial Number"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')

                offset = offset + length + 1
                length = int.from_bytes(eeprom_data[offset:offset+1], "big")
                length &= 0x3f
                self.boardinfo["Board Part Number"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')

                offset = offset + length + 1
                length = int.from_bytes(eeprom_data[offset:offset+1], "big")
                length &= 0x3f
                #/* Skip FRU File ID */
                offset = offset + length + 1
                length = int.from_bytes(eeprom_data[offset:offset+1], "big")
                length &= 0x3f
                self.boardinfo["Board Revision"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')
            except IOError:
                logging.error(f"Onboard {eeprom.Name} Eeprom read failed!")
            finally:
                i2c.close()

        for ps in pm_powersensor:
            try:
                match ps.part_name:
                    case 'INA226':
                        ps._sensor = INA226(ps.i2c_address, ps.i2c_bus)
                    case 'INA700' | 'INA745':
                        ps._sensor = INA7XX(ps.i2c_address, ps.i2c_bus)
                    case _:
                        ps._sensor = None
            except Exception as e:
                print(e)

            if ps._sensor is None:
                self.logger.error(f"Sensor({ps.name}) device initialization is Failed.")
            else:
                if ps._sensor.initSensor(ps.maximum_current, ps.shunt_resistor, ps.phase_multiplier):
                    self.power_sensors.append(ps)
                    self.logger.debug(f"Sensor({ps.name}) device initialization Successfully.")
                else:
                    self.logger.error(f"Sensor({ps.name}) device is not found !")

        directory = os.path.dirname(RAFT_DIR + '.raft/')
        if directory and not os.path.exists(directory):
            os.makedirs(directory)
        bootvoltage_path = os.path.join(directory, 'bootvoltage')

        db = dbm.dumb.open(bootvoltage_path, 'c')
        for v in pm_voltages:
            v._output = None
            match v.part_name:
                case 'IR35215'| 'IRPS5401' | 'IR38164' | 'IR38064' | 'IR38060' | 'TPS53681' | 'TPS546B24A' | 'TPS546D24A' | 'TPS544B25':
                    v._output = PMBusRegulator(device_name=v.part_name, device_path=v.i2c_bus, device_address=v.i2c_address, page=v.page_select, pmbus_vout_mode=v.pmbus_vout_mode, phase=v.phase)
                case 'MPM54322' | 'MPM54522':
                    v._output = MPSRegulator(device_path=v.i2c_bus, device_address=v.i2c_address, page=v.page_select, phase=v.phase, fb_ratio=v.fb_ratio)
                case _:
                    v._output = None #DefaultRegulator(device_path=v.i2c_bus, device_address=v.i2c_address)
            if v._output is None:
                self.logger.error(f"Voltage regulator for {v.name} is not defined.")
            else:
                self.voltages.append(v)
                self.logger.debug(f"Regulator Output({v.name}) device initialization Successfully.")
                for key, value in db.items():
                    voltage_name = key.decode('utf-8')
                    if voltage_name == v.name:
                        self.SetVoltage(v._output, float(value))
        db.close()
        self.logger.info("Inside PMIC Constructor")

    @staticmethod
    def GetLogger():
        log_level = logging.ERROR
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            logger.handler_set = True
            logger.disabled = False
        return logger

    def GetPythonLogLevels(self):
        return get_python_log_levels()

    def SetServerLogLevel(self, PythonLogLevel):
        self.logger.debug(f"PythonLogLevel = {PythonLogLevel}")
        LogLevelsDict = get_python_log_levels()
        if PythonLogLevel == LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def EnableVoltage(self, o):
        self.logger.debug(f"EnableVoltage({o.addr}@{o.i2c.devpath})")
        o.enable_output()

    def DisableVoltage(self, o):
        self.logger.debug(f"DisableVoltage({o.addr}@{o.i2c.devpath})")
        o.shutdown_output()

    def GetRegulatorAll(self, o):
        self.logger.debug(f"GetRegulatorAll({o.addr}@{o.i2c.devpath})")
        data = {}
        telemetry = o.read_telemetry_all()
        data['vin'] = telemetry[0]
        data['iin'] = telemetry[1]
        data['vout'] = telemetry[2]
        data['iout'] = telemetry[3]
        data['temp'] = telemetry[4]
        data['pout'] = telemetry[5]
        data['pin'] = telemetry[6]
        return data

    def GetVoltage(self, o):
        self.logger.debug("GetVoltage(0x{0:02x}@{1})".format(o.addr, o.i2c.devpath))
        data = {}
        data['Voltage'] = round(o.read_voltage(), 4)
        return data

    def SetVoltage(self, o, val):
        self.logger.debug("SetVoltage(0x{0:02x}@{1}, {2})".format(o.addr, o.i2c.devpath, val))
        o.set_voltage(val)

    def GetPowerSensorConf(self, s):
        self.logger.debug("GetPowerSensorConf(0x{0:02x}@{1})".format(s.addr, s.i2c.devpath))
        data = {}
        registers = s.readRegisterValues()
        data['Configuration'] = '0x{0:04x}'.format(registers[INA226.CONFIGURATION])
        data['Shunt_Voltage'] = '0x{0:04x}'.format(registers[INA226.SHUNT_VOLTAGE])
        data['Bus_Voltage'] =   '0x{0:04x}'.format(registers[INA226.BUS_VOLTAGE])
        data['Power'] = '0x{0:04x}'.format(registers[INA226.POWER])
        data['Current'] = '0x{0:04x}'.format(registers[INA226.CURRENT])
        data['Calibration'] = '0x{0:04x}'.format(registers[INA226.CALIBRATION])
        data['Mask_Enable'] = '0x{0:04x}'.format(registers[INA226.MASK_ENABLE])
        data['Alert_Limit'] = '0x{0:04x}'.format(registers[INA226.ALERT_LIMIT])
        return data

    def SetPowerSensorConf(self, s, data):
        self.logger.debug("SetPowerSensorConf(0x{0:02x}@{1})".format(s.addr, s.i2c.devpath))
        list_regs = []
        for reg in data:
            for k, v in reg.items():
                temp = None
                match k:
                    case 'Configuration':
                        temp = {INA226.CONFIGURATION : v}
                    case 'Calibration':
                        temp = {INA226.CALIBRATION : v}
                    case 'Mask_Enable':
                        temp = {INA226.MASK_ENABLE : v}
                    case 'Alert_Limit':
                        temp = {INA226.ALERT_LIMIT : v}
                if temp is not None:
                    list_regs.append(temp)
        s.writeRegisterValues(list_regs)

    def GetSensorValues(self, s):
        self.logger.debug("GetSensorValues(0x{0:02x}@{1})".format(s.addr, s.i2c.devpath))
        vbus = s.getBusVoltage()
        vshunt =  s.getShuntVoltage()
        current = s.getCurrent()
        power = s.getPower()
        return vbus, vshunt, current, power

    def __del__(self):
        self.logger.info("Inside PMIC Destructor")
